<?php  
/* 
Plugin Name: UserCredits from Outerbridge (shared on wplocker.com)
Plugin URI: http://ofrm.co.uk 
Description: Use and manage user credits to access site content.  <strong>ALERT: Major Changes in UserCredits Version 4.0.  You must review all settings and shortcodes.</strong>
Author: Outerbridge
Version: 4.1
Author URI: http://outerbridge.co.uk
License: This plugin is copyright Outerbridge Ltd.
*/  

/**
 *
 * Changelog
 *
 *	v4.1	140723	added $_POST, $_REQUEST, $_GET options, improved set-up page and help file, and tidied comments, fixed pagination on page access admin page
 *
 *	v4.0.1	140705	fixed the set-up bug which called obr_now() rather than obr_datetime()
 *	v4.0	140702	complete re-write with multiple new features and retired previous shortcodes
 *	v3.1.2	140226	added [days] and [hours] to grace time period - not just [min].  Made [s] compatable with each
 *	v3.1.1	131210	fixed initial balance table not being correctly created
 *	v3.1	131206	fixed "You cannot add further credits that quickly" message not displaying.  Also changed viewtime and addupdatelock to use WP options rather than session variables.
 *	v3.0	131129	general code tidying (e.g. admin menus) and made translation-ready
 *	v2.3	131120	added credits expiry functionality (not a public release)
 *	v2.2 	130725	added visual shortcodes for the four shortcodes
 *	v2.1.3	130227	corrected inability to add new bundles if admin has deleted them all
 *	v2.1.2	130211	corrected issue with insufficient balance levels
 *	v2.1.1	121016	corrected incorrect table prefix on user balances and this page
 *	v2.1.0	120823	added ability to change the auto increase credits variable name from "addvalue" to a custom value
 *	v2.0.0	120615	reworked plugin to include custom messages
 *	v1.4.0	120130	corrected bug which prevented using zero as default number of credits
 *	v1.3.1	120116	made addupdatelock and viewtime sessions unique so that they don't conflict with each other on different pages
 *	v1.3	120114	added addupdatelock on increasing credits (in minutes) with a default of 60 minutes and viewtime on reducing credits (in minutes) with a default of 10 minutes 
 *	v1.2	updated for refreshing on increasing credits
 *	v1.1	updated version including Siddharth's comments
 *	v1.0	first version submitted to code canyon
 *
 */

global $wpdb;
	
// define the table names to be used
global $obr_table_name;
$obr_table_name = "{$wpdb->prefix}obr_credits";
global $obr_access_table_name;
$obr_access_table_name = "{$wpdb->prefix}obr_credits_access";
global $obr_admin_table_name;
$obr_admin_table_name = "{$wpdb->prefix}obr_credits_admin";
global $obr_transaction_log_table_name;
$obr_transaction_log_table_name = "{$wpdb->prefix}obr_credits_transaction_log";

// if you must change this - add new options to the end of the following arrays, otherwise the existing data will not be correct
global $expiryvalues;
global $expiryvaluesfordb;
$expiryvalues      = array(__('Unlimited', 'uc'), __('1 Week', 'uc'), __('1 Month', 'uc'), __('3 Months', 'uc'), __('6 Months', 'uc'), __('1 Year'));
$expiryvaluesfordb = array('',                    '+1 week',          '+1 month',          '+3 months',          '+6 months',          '+1 year');

// define the table name to be used
if (!class_exists("obr_usercredits")){
	class obr_usercredits{
		// version
		public $obr_credits_version = "4.1";

		// admin key for bundles - set to 2 for backwards compatibility
		public $obr_bundle_key = 2;
		
		//constructor
		function obr_usercredits(){
			$this->__construct();
		}
	
		function __construct(){
			// this hook will cause our creation function to run when the plugin is activated
			register_activation_hook(__FILE__, array($this, 'obr_plugin_init'));		
			add_action('plugins_loaded', array($this, 'obr_plugin_update_check'));
			add_action('plugins_loaded', array($this, 'obr_internationalisation'));
			add_action('admin_head', array($this, 'obr_add_menu_icons_styles'));
			add_action('admin_menu', array($this, 'obr_admin_menu'));
			add_action('wp_enqueue_scripts', array($this, 'obr_scripts_and_styles'));
			add_action('admin_enqueue_scripts', array($this, 'obr_admin_scripts_and_styles'));

			add_action('init', array($this, 'obr_uc_shortcode_button_init'));
			add_action('admin_head', array($this, 'obr_admin_head'));

			add_action('admin_bar_menu', array($this, 'obr_admin_bar_item'), 999);
			add_action('init', array($this, 'obr_init_post_actions'), 1);

			add_action('user_register', array($this, 'obr_new_user_registered'));
			// before delete so delete_user rather than deleted_user
			add_action('delete_user', array($this, 'obr_user_deleted')); 

			add_shortcode('obr_credits_added_email_alert', array($this, 'obr_credits_added_email_alert'));
			add_shortcode('obr_low_credits_email_alert', array($this, 'obr_low_credits_email_alert'));
			add_shortcode('obr_balance_update_email_alert', array($this, 'obr_balance_update_email_alert'));

			add_shortcode('obr_page_button', array($this, 'obr_page_button'));
			add_shortcode('obr_check_credits', array($this, 'obr_check_credits'));
			add_shortcode('obr_display_balance', array($this, 'obr_display_balance'));
			add_shortcode('obr_add_credits', array($this, 'obr_add_credits'));

			// add cron schedule in
			register_activation_hook(__FILE__, array($this, 'obr_schedule_activation'));
			add_filter('cron_schedules', array($this, 'obr_add_to_wp_cron_options'));

			add_action('obr_event_hook_1', array($this, 'obr_periodic_credits_review'));

			add_action('obr_event_hook_2', array($this, 'obr_credits_added_email_alert'));
			add_action('obr_event_hook_3', array($this, 'obr_low_credits_email_alert'));
			add_action('obr_event_hook_4', array($this, 'obr_balance_update_email_alert'));
			register_deactivation_hook(__FILE__, array($this, 'obr_schedule_deactivation'));
		}
	
		function obr_plugin_init(){
			// add in a version number
			add_option('obr_credits_version', $this->obr_credits_version);
			$this->obr_create_tables('init');
		}

		function obr_plugin_update_check(){
			// check if there's an update
			$installed_ver = get_option('obr_credits_version');
			if($installed_ver != $this->obr_credits_version){
				$this->obr_create_tables('update');
				// update specifics go here
				require_once(ABSPATH.'wp-admin/includes/upgrade.php');
				update_option('obr_credits_version', $this->obr_credits_version);
				?>
				<div id="message" class="updated fade">
					<p>
						<strong>
							<?php printf(__('UserCredits from Outerbridge Updated to Version %s', 'uc'), $this->obr_credits_version); ?>
						</strong>
					</p>
					<p>
						<strong>
							<?php _e('ALERT: Major Changes in UserCredits Version 4.0.  You must review all settings and shortcodes.', 'ucipn'); ?>
						</strong>
					</p>
				</div>
				<?php
			}
		}

		function obr_create_tables($process){
			// if $process is 'init' then we need to check the existence of the tables, if it's an update then we don't
			global $wpdb;
			global $obr_table_name;
			global $obr_access_table_name;
			global $obr_admin_table_name;
			global $obr_transaction_log_table_name;
	
			$mysql = "CREATE TABLE $obr_table_name (
				fld_ref int(11) NOT NULL AUTO_INCREMENT,
				ID int(11) NOT NULL,
				fld_credits int(11) NOT NULL DEFAULT 0,
				fld_bundle_type smallint(6) NOT NULL DEFAULT 0,
				fld_expiry_date datetime,
				last_credit datetime,
				last_reminder datetime,
				last_update timestamp,
				UNIQUE KEY fld_ref (fld_ref),
				UNIQUE KEY ID (ID)
			);";
			if(($process == 'update') || (($process == 'init') && ($wpdb->get_var("SHOW TABLES LIKE '$obr_table_name';") != $obr_table_name))){
				require_once(ABSPATH.'wp-admin/includes/upgrade.php');
				dbDelta($mysql);
			}
			$mysql = "CREATE TABLE $obr_access_table_name (
				fld_ref int(11) NOT NULL AUTO_INCREMENT,
				ID int(11) NOT NULL DEFAULT 0,
				page int(11) NOT NULL DEFAULT 0,
				pageexpiry datetime,
				UNIQUE KEY fld_ref (fld_ref)
			);";
			if(($process == 'update') || (($process == 'init') && ($wpdb->get_var("SHOW TABLES LIKE '$obr_access_table_name';") != $obr_access_table_name))){
				require_once(ABSPATH.'wp-admin/includes/upgrade.php');
				dbDelta($mysql);
			}
			$mysql = "CREATE TABLE $obr_admin_table_name (
				fld_ref int(11) NOT NULL AUTO_INCREMENT,
				fld_key mediumint(9) NOT NULL,
				fld_value decimal(19,2) NOT NULL DEFAULT 0,
				fld_value2 mediumint(9) NOT NULL DEFAULT 0,
				fld_expiry_period smallint(6) NOT NULL DEFAULT 0,
				UNIQUE KEY fld_ref (fld_ref)
			);";
			if(($process == 'update') || (($process == 'init') && ($wpdb->get_var("SHOW TABLES LIKE '$obr_admin_table_name';") != $obr_admin_table_name))){
				require_once(ABSPATH.'wp-admin/includes/upgrade.php');
				dbDelta($mysql);
			}
			if(($process == 'init') && ($wpdb->get_var("SELECT COUNT(fld_ref) FROM $obr_admin_table_name;") == 0)){
				// now put in the default admin values - use fld_key of 2 for backwards compatability
				$new_admin_rows = $wpdb->insert($obr_admin_table_name, array('fld_key' => $this->obr_bundle_key, 'fld_value' => 1.99, 'fld_value2' => 10));
				$new_admin_rows = $wpdb->insert($obr_admin_table_name, array('fld_key' => $this->obr_bundle_key, 'fld_value' => 2.99, 'fld_value2' => 20));
				$new_admin_rows = $wpdb->insert($obr_admin_table_name, array('fld_key' => $this->obr_bundle_key, 'fld_value' => 3.99, 'fld_value2' => 30));
			}
			$mysql = "CREATE TABLE $obr_transaction_log_table_name (
				fld_ref int(11) NOT NULL AUTO_INCREMENT,
				ID int(11) NOT NULL DEFAULT 0,
				credits int(11) NOT NULL DEFAULT 0,
				credit_time datetime,
				transaction_id VARCHAR(100),
				UNIQUE KEY fld_ref (fld_ref)
			);";
			if(($process == 'update') || (($process == 'init') && ($wpdb->get_var("SHOW TABLES LIKE '$obr_transaction_log_table_name';") != $obr_transaction_log_table_name))){
				require_once(ABSPATH.'wp-admin/includes/upgrade.php');
				dbDelta($mysql);
			}
		}
		
		function obr_init_post_actions(){
			require_once('user-credits-init-actions.php');
		}

		function obr_add_menu_icons_styles(){
			?>
			<style>
			#adminmenu #toplevel_page_obr-online-exercises div.wp-menu-image:before {
				content: '\f307';
				display: inline-block;
				-webkit-font-smoothing: antialiased;
				font: normal 20px/1 'dashicons';
				vertical-align: top;
			}
			</style>
			<?php
		}

		function obr_admin_menu(){
			// create the admin menus
			add_menu_page('UserCredits Admin', 'UserCredits', 'read', 'obr-usercredits', array($this, 'obr_my_usercredits_admin'));
			add_submenu_page('obr-usercredits', __('My UserCredits', 'uc'), __('My UserCredits', 'uc'), 'read', 'obr-usercredits');
			add_submenu_page('obr-usercredits', __('Set-Up', 'uc'), __('Set-Up', 'uc'), 'manage_options', 'obr-set-up', array($this, 'obr_set_up_admin'));
			add_submenu_page('obr-usercredits', __('Bundles', 'uc'), __('Bundles', 'uc'), 'manage_options', 'obr-bundles', array($this, 'obr_bundles_admin'));
			add_submenu_page('obr-usercredits', __('User Balances', 'uc'), __('User Balances', 'uc'), 'manage_options', 'obr-usercredits-balances', array($this, 'obr_balances_admin'));
			add_submenu_page('obr-usercredits', __('Page Access', 'uc'), __('Page Access', 'uc'), 'manage_options', 'obr-usercredits-page-access', array($this, 'obr_page_access_admin'));
			add_submenu_page('obr-usercredits', __('Balance Resets', 'uc'), __('Balance Resets', 'uc'), 'manage_options', 'obr-usercredits-balance-resets', array($this, 'obr_balance_resets_admin'));
			add_submenu_page('obr-usercredits', __('Transaction Log', 'uc'), __('Transaction Log', 'uc'), 'read', 'obr-transaction-log', array($this, 'obr_transaction_log_admin'));
			add_submenu_page('obr-usercredits', __('Custom Messages', 'uc'), __('Custom Messages', 'uc'), 'manage_options', 'obr-custom-messages', array($this, 'obr_custom_messages_admin'));
			add_submenu_page('obr-usercredits', __('Help & Usage', 'uc'), __('Help & Usage', 'uc'), 'manage_options', 'obr-usercredits-help-usage', array($this, 'obr_help_usage_admin'));
		}

		function obr_internationalisation(){
			// willkommen tout le monde...
			load_plugin_textdomain('uc', false, dirname(plugin_basename(__FILE__)).'/languages/');
		}
	
		function obr_my_usercredits_admin(){
			require_once('user-credits-my-usercredits-admin.php');
			require_once('user-credits-my-usercredits-noerror.php');
		}

		function obr_set_up_admin(){
			require_once('user-credits-set-up-admin.php');
		}

		function obr_bundles_admin(){
			require_once('user-credits-bundles-admin.php');
		}

		function obr_balances_admin(){
			require_once('user-credits-user-balances-admin.php');
		}

		function obr_page_access_admin(){
			require_once('user-credits-page-access-admin.php');
		}

		function obr_balance_resets_admin(){
			require_once('user-credits-balance-resets-admin.php');
		}

		function obr_transaction_log_admin(){
			require_once('user-credits-transaction-log-admin.php');
		}

		function obr_custom_messages_admin(){
			require_once('user-credits-custom-messages-admin.php');
		}

		function obr_help_usage_admin(){
			require_once('user-credits-help-usage-admin.php');
		}

		function obr_scripts_and_styles(){
			wp_enqueue_style(
				'user-credits-style',
				plugins_url('/user-credits.css', __FILE__)
			);
		}

		function obr_admin_scripts_and_styles() {
			wp_register_style(
				'user-credits-admin-style', 
				plugins_url('/user-credits-admin.css', __FILE__)
			);
			wp_enqueue_style('user-credits-admin-style');
		}

		function obr_admin_head(){
			// function to add localisation for the shortcode buttons
			$displaypagecost = intval(get_option('obr_pagecost'));
			$displaypageexpiry = intval(get_option('obr_pageexpiry'));
			$displaycm_db_singular = esc_attr(get_option('obr_cm_db_singular'));
			if (intval(strlen($displaycm_db_singular)) == 0){
				$displaycm_db_singular = __('You have 1 credit.', 'uc');
			}
			$displaycm_db_plural = esc_attr(get_option('obr_cm_db_plural'));
			if (intval(strlen($displaycm_db_plural)) == 0){
				$displaycm_db_plural = __('You have %d credits.', 'uc');
			}
			?>
			<!-- TinyMCE Shortcode Plugin -->
			<script type='text/javascript'>
			var shortcode_variables = {
				'pagecost': '<?php echo $displaypagecost; ?>',
				'pageexpiry': '<?php echo $displaypageexpiry; ?>',
				'cm_db_singular': '<?php echo $displaycm_db_singular; ?>',
				'cm_db_plural': '<?php echo $displaycm_db_plural; ?>',
			};
			</script>
			<!-- TinyMCE Shortcode Plugin -->
			<?php
		}

		function obr_datetime(){
			// get a standard datetime string
			$datetime = new DateTime();
			$tz = get_option('timezone_string');
			if (strlen($tz) == 0){
				$tz = 'Europe/London';
			}
			$datetime = $datetime->setTimezone(new DateTimezone($tz));
			$formatteddatetime = $datetime->format('Y-m-d H:i:s');
			return $formatteddatetime;
		}

		function obr_new_user_registered($userid){
			// a new member has been added so increase their credits balance
			global $wpdb;
			global $obr_table_name;
			global $obr_transaction_log_table_name;

			$defaultcredits = intval(get_option('obr_defaultcredits'));
			
			$q = $wpdb->insert($obr_table_name, 
								array('ID' => $userid, 'fld_credits' => $defaultcredits, 'last_credit' => $this->obr_datetime(), 'last_reminder' => $this->obr_datetime()), 
								array('%d', '%d', '%s', '%s')
							);
			$transactionlogcomment = sprintf(__('User %d registered with %d credits.', 'uc'), $userid, $defaultcredits);
			$q2 = $this->obr_log_transaction($userid, $defaultcredits, $transactionlogcomment);

			// now we process the obr_user_registered hook
			$hookargs = array(
						'userid' => $userid,
						'credits' => $defaultcredits,
						'txn_id' => $transactionlogcomment 
					);
			do_action('obr_user_registered', $hookargs);
		}

		function obr_user_deleted($userid){
			// user deleted, so delete their records
			global $wpdb;
			global $obr_table_name;
			global $obr_transaction_log_table_name;

			$balanceondeletion = $this->obr_balance($userid);
			$user_info = get_userdata($userid);
			$display_name = $user_info->display_name;
			$transactionlogcomment = sprintf(__('User %d (%s) deleted with %d credits.', 'uc'), $userid, $display_name, $balanceondeletion);
			$q = $this->obr_log_transaction($userid, -$balanceondeletion, $transactionlogcomment);
			$q2 = $wpdb->delete($obr_table_name,
								array('ID' => $userid),
								array('%d')
							);
		}

		function obr_credits_added_email_alert(){
			// this needs to run every day to send out email reminders as necessary...
			global $wpdb;
			global $obr_table_name;

			$credit_added_emails = get_option('obr_credit_added_emails');
			$credit_added_emails = (bool)$credit_added_emails;
			if ($credit_added_emails){
				// reminders are enabled (false means send no emails)
				$q = $wpdb->get_results("select u.ID, u.user_email, u.display_name, o.fld_credits, o.last_credit, o.last_reminder 
										 from {$wpdb->prefix}users u 
										 inner join $obr_table_name o on o.ID = u.ID
										 order by ID;");
				if ($q){
					foreach ($q as $row){
						// now we check to see whether we need to send an email...
						$userid = $row->ID;
						$credits = $row->fld_credits;
						$lastcredit = date("Y-m-d H:i:s", strtotime($row->last_credit));
						$lastreminder = date("Y-m-d H:i:s", strtotime($row->last_reminder));
						if ($lastreminder < $lastcredit){
							// we need to send a reminder
							$email = $row->user_email;

							$subject = sprintf(__('Credits have been added to your account.  You have %d credits available', 'uc'), $credits);

							ob_start();
							include 'email-templates/email-credits-added-html.html';
							$message = ob_get_clean();

							$headers = "MIME-Version: 1.0\r\n";
							$headers .= "Content-Type: text/html; charset=ISO-8859-1\r\n";
							if (wp_mail($email, $subject, $message, $headers)){
								$q3 = $wpdb->update($obr_table_name, 
													array('last_reminder' => $this->obr_datetime()), 
													array('ID' => $userid),
													array('%s'),
													array('%d')
												);
							}
						}
					}
				}
			}
		}

		function obr_low_credits_email_alert(){
			// this runs every day to send out email reminders of low credit balances as necessary...
			global $wpdb;
			global $obr_table_name;

			$low_credit_emails = intval(get_option('obr_low_credit_emails'));
			if ($low_credit_emails > 0){
				// reminders are enabled (zero means send no reminders)
				$q = $wpdb->get_results("select u.ID, u.user_email, u.display_name, o.fld_credits, o.fld_expiry_date, o.last_reminder 
										 from {$wpdb->prefix}users u 
										 inner join $obr_table_name o on o.ID = u.ID
										 order by ID;");
				if ($q){
					foreach ($q as $row){
						// now we check to see whether we need to send an email...
						$userid = $row->ID;
						$credits = $row->fld_credits;
						if ($credits < $low_credit_emails){
							// we need to send a reminder
							$email = $row->user_email;

							$subject = sprintf(__('Your credits are running low.  You only have %d credits available', 'uc'), $credits);

							ob_start();
							include 'email-templates/email-low-credits-html.html';
							$message = ob_get_clean();

							$headers = "MIME-Version: 1.0\r\n";
							$headers .= "Content-Type: text/html; charset=ISO-8859-1\r\n";

							if (wp_mail($email, $subject, $message, $headers)){
								$q2 = $wpdb->update($obr_table_name, 
													array('last_reminder' => $this->obr_datetime()), 
													array('ID' => $userid),
													array('%s'),
													array('%d')
												);
							}
						}
					}
				}
			}
		}

		function obr_balance_update_email_alert(){
			// weekly email updating users as to their balance...
			global $wpdb;
			global $obr_table_name;

			$balance_update_emails = get_option('obr_balance_update_emails');
			$balance_update_emails = (bool)$balance_update_emails;
			if ($balance_update_emails){
				// reminders are enabled (false means send no emails)
				$q = $wpdb->get_results("select u.ID, u.user_email, u.display_name, o.fld_credits, o.fld_expiry_date, o.last_reminder 
										 from {$wpdb->prefix}users u 
										 inner join $obr_table_name o on o.ID = u.ID
										 order by ID;");
				if ($q){
					foreach ($q as $row){
						// now we check to see whether we need to send an email...
						$userid = $row->ID;
						$credits = $row->fld_credits;

						if ($credits > 0){
							// we need to send an email
							$email = $row->user_email;

							$subject = sprintf(__('You have %d credits available', 'uc'), $credits);

							ob_start();
							include 'email-templates/email-balance-update-html.html';
							$message = ob_get_clean();

							$headers = "MIME-Version: 1.0\r\n";
							$headers .= "Content-Type: text/html; charset=ISO-8859-1\r\n";

							if (wp_mail($email, $subject, $message, $headers)){
								$q2 = $wpdb->update($obr_table_name, 
													array('last_reminder' => $this->obr_datetime()), 
													array('ID' => $userid),
													array('%s'),
													array('%d')
												);
							}
						}
					}
				}
			}
		}

		function obr_page_access_check($pageid){
			// function to check page access
			// return array with pageid if have access, 0 if not, and -1 if page not available, and pageexpiry if relevant
			global $wpdb;
			global $obr_access_table_name;
			if (get_post_status($pageid) == 'publish'){
				// this page exists so now check whether the current user has access to it...
				$userid = get_current_user_id();
				$userhasaccess = intval($wpdb->get_var("select page from $obr_access_table_name where ID=$userid and page=$pageid;"));
				if ($pageid == $userhasaccess){
					// user already has access to this page
					// check for expiry of page
					$pageexpiry = esc_attr($wpdb->get_var("select pageexpiry from $obr_access_table_name where ID=$userid and page=$pageid;"));
					$formattedpageexpiry = new DateTime($pageexpiry, new DateTimezone(get_option('timezone_string')));
					$now = new DateTime();
					$now = $now->setTimezone(new DateTimezone(get_option('timezone_string')));
					$diff = $now->diff($formattedpageexpiry);
					$interval = $diff->i;
					$seconds = intval($diff->days*86400 + $diff->h*3600 + $diff->i*60 + $diff->s);
					if ($diff->invert){
						// this is a negative
						$seconds = -$seconds;
					}
					if ($seconds < 0){
						// this page has expired so remove access
						$q = $wpdb->delete($obr_access_table_name, 
											array('ID' => $userid, 'page' => $pageid), 
											array('%d', '%d'));
						$returnstring = array('page' => 0, 'pageexpiry' => null);
					} else {
						// page has not expired
						$returnstring = array('page' => $pageid, 'pageexpiry' => $pageexpiry);
					}
				} else {
					// user doesn't have access to this page
					$returnstring = array('page' => 0, 'pageexpiry' => null);
				}
			} else {
				$returnstring = array('page' => -1, 'pageexpiry' => null);
			}
			return $returnstring;
		}

		function obr_page_button($atts, $content= null){
			// shortcode to create a button to a protected page
			extract(shortcode_atts(array('pageid' => 0, 
										 'access' => null, 
										 'noaccess' => null
										), $atts));
			if ($access === null || strlen($access) == 0){
				$cm_pb_access = esc_attr(get_option('obr_cm_pb_access'));
				if (intval(strlen($cm_pb_access)) == 0){
					$access = __('Open Page', 'uc');
				} else {
					$access = $cm_pb_access;
				}
			}
			if ($noaccess === null || strlen($noaccess) == 0){
				$cm_pb_noaccess = esc_attr(get_option('obr_cm_pb_noaccess'));
				if (intval(strlen($cm_pb_noaccess)) == 0){
					$noaccess = __('Page Preview', 'uc');
				} else {
					$noaccess = $cm_pb_noaccess;
				}
			}

			$viewpage = $this->obr_page_access_check($pageid);
			if ($viewpage['page'] > 0){
				$returnstring = $this->obr_create_html_button($access, 'green', $pageid);
			} elseif ($viewpage['page'] == 0){
				$returnstring = $this->obr_create_html_button($noaccess, 'red', $pageid);
			} else {
				$cm_pb_no_pageid = esc_attr(get_option('obr_cm_pb_no_pageid'));
				if (intval(strlen($cm_pb_no_pageid)) == 0){
					$returnstring = sprintf(__('The page with id number %d does not exist.', 'uc'), $pageid);
				} else {
					$returnstring = sprintf($cm_pb_no_pageid, $pageid);
				}
			}
			return $returnstring.$content;
		}

		function obr_create_html_button($wording = null, $colour = 'red', $id = 0){
			// create an html button for the page
			$pageurl = get_permalink($id);
			$returnstring = '<form action="'.$pageurl.'">';
			$returnstring .= '<input type="submit" value="'.$wording.'" class="page-button '.$colour.'-button" />';
			$returnstring .= '</form>';
			return $returnstring;
		}

		function obr_check_credits($atts, $content = null){
			// find out whether the user has access to this page and react as appropriate
			global $wpdb;
			global $obr_table_name;
			global $obr_access_table_name;
			global $obr_transaction_log_table_name;
			$returnstring = '';

			$accesscost = intval(get_option('obr_pagecost'));
			$args = shortcode_atts(array('accesscost' => $accesscost, 'expiryminutes' => -1), $atts);
			$accesscost = intval($args['accesscost']);
			$expiryminutes = intval($args['expiryminutes']);

			$pageexpiry = intval(get_option('obr_pageexpiry'));
			if ($expiryminutes >= 0){
				// we have been given the expiry parameter, so use it in preference to the global settings
				$pageexpiry = $expiryminutes;
			}

			$userid = get_current_user_id();
			if ($userid){
				if (isset($_POST['submit'])){
					// first see whether we need to make a page purchase
					$accesscost = intval($_POST['pagecost']);
					$page = intval($_POST['page']);
					$userid = get_current_user_id();

					$alreadypurchasedthispage = $wpdb->get_var("select page from $obr_access_table_name where ID=$userid and page=$page;");
					$q = '';
					$q2 = '';
					$q3 = '';
					if (!$alreadypurchasedthispage){
						// not purchased this previously so let's allow access and charge for it

						// first find out about the expiry date
						$expiresat = null;
						$expiresattxnlog = sprintf(__('Purchased indefinite access to page %d', 'uc'), $page);
						if ($pageexpiry > 0){
							// we need to add the number of minutes to the current time...
							$expiresatdate = new DateTime();
							$expiresatdate = $expiresatdate->setTimezone(new DateTimezone(get_option('timezone_string')));
							$expiresatdate->format('Y-m-d H:i:s');
							$expiresatdate->add(new DateInterval('PT'.$pageexpiry.'M'));
							$expiresat = $expiresatdate->format('Y-m-d H:i:s');
							$expiresattxnlog = sprintf(__('Purchased access to page %d until %s', 'uc'), $page, $expiresat);
						}
						if (is_null($expiresat)){
							$q = $wpdb->insert($obr_access_table_name,
												array('ID' => $userid, 'page' => $page)
											);
						} else {
							$q = $wpdb->insert($obr_access_table_name,
												array('ID' => $userid, 'page' => $page, 'pageexpiry' => $expiresat)
											);
						}
						$q2 = $this->obr_log_transaction($userid, -$accesscost, $expiresattxnlog);
						$oldbalance = $this->obr_balance($userid);
						$newbalance = $oldbalance - $accesscost;
						$q3 = $wpdb->update($obr_table_name, 
											array('fld_credits' => $newbalance), 
											array('ID' => $userid)
										);
					}
					if ($q && $q2 && $q3){
						$cm_cc_purchase_singular = esc_attr(get_option('obr_cm_cc_purchase_singular'));
						if (intval(strlen($cm_cc_purchase_singular)) == 0){
							$singular = __('Thank you for your purchase. You now have 1 credit.', 'uc');
						} else {
							$singular = $cm_cc_purchase_singular;
						}
						$cm_cc_purchase_plural = esc_attr(get_option('obr_cm_cc_purchase_plural'));
						if (intval(strlen($cm_cc_purchase_plural)) == 0){
							$plural = __('Thank you for your purchase. You now have %d credits.', 'uc');
						} else {
							$plural = $cm_cc_purchase_plural;
						}
						$returnstring = '<p>'.sprintf(_n($singular, $plural, $newbalance, 'uc'), $newbalance).'</p>';

						// now we allow the hook to be used
						$hookargs = array(
									'userid' => $userid,
									'credits' => $accesscost, 
									'txn_id' => $expiresattxnlog, 
									'newbalance' => $newbalance
								);
						if ($pageexpiry > 0){
							$hookargs['expiry'] = $expiresat;
						} else {
							$hookargs['expiry'] = null;
						}
						do_action('obr_credits_redeemed', $hookargs);
					} else {
						$cm_cc_purchase_problem = esc_attr(get_option('obr_cm_cc_purchase_problem'));
						if (intval(strlen($cm_cc_purchase_problem)) == 0){
							$problem = __('There was a problem with your purchase.  Please contact the administrator.', 'uc');
						} else {
							$problem = $cm_cc_purchase_problem;
						}
						$returnstring .= '<p>'.$problem.'</p>';
					}
				}
				$pageid = get_the_ID();
				$viewpage = $this->obr_page_access_check($pageid);

				$preview = '';
				$failurepresent = strpos($content, '[obr_end_preview]');
				if ($failurepresent > 0 || $failurepresent === 0){
					list($if, $else) = explode('[obr_end_preview]', $content, 2);
					$content = $else;
					$preview = $if;
				}

				if ($viewpage['page']){
					// page is already available
					$returnstring .= $content;
				} elseif ($viewpage['page'] == 0){
					// page is not available
					$returnstring .= $this->obr_no_access_yet($pageid, $accesscost);
					$returnstring .= $preview;
				} else {
					// use custom message 3.3 here
					$cm_pb_no_pageid = esc_attr(get_option('obr_cm_pb_no_pageid'));
					if (intval(strlen($cm_pb_no_pageid)) == 0){
						$returnstring .= sprintf(__('The page with id number %d does not exist.', 'uc'), $pageid);
					} else {
						$returnstring .= sprintf($cm_pb_no_pageid, $pageid);
					}
				}
			} else {
				$cm_cc_not_logged_in = esc_attr(get_option('obr_cm_cc_not_logged_in'));
				if (intval(strlen($cm_cc_not_logged_in)) == 0){
					$returnstring .= __('You need to be logged in to purchase and view this page.', 'uc');
				} else {
					$returnstring .= $cm_cc_not_logged_in;
				}
			}
			return $returnstring;
		}

		function obr_no_access_yet($pageid = 0, $pagecost = 0){
			// function to display a buy now button
			$userid = get_current_user_id();
			$returnstring = '';
			if ($userid){
				$credits = $this->obr_balance($userid);
				$returnstring .= '<div class="purchase-box">';

				// concatenate custom messages 2.1 and 2.2 with 4.5 and 4.6
				$cm_db_singular = esc_attr(get_option('obr_cm_db_singular'));
				if (intval(strlen($cm_db_singular)) == 0){
					$singular = __('You have 1 credit.', 'uc');
				} else {
					$singular = $cm_db_singular;
				}
				$cm_db_plural = esc_attr(get_option('obr_cm_db_plural'));
				if (intval(strlen($cm_db_plural)) == 0){
					$plural = __('You have %d credits.', 'uc');
				} else {
					$plural = $cm_db_plural;
				}
				$messagestring1 = sprintf(_n($singular, $plural, $credits, 'uc'), $credits);

				$cm_cc_need_credit_singular = esc_attr(get_option('obr_cm_cc_need_credit_singular'));
				if (intval(strlen($cm_cc_need_credit_singular)) == 0){
					$singular = __('You need 1 credit to access this page.', 'uc');
				} else {
					$singular = $cm_cc_need_credit_singular;
				}
				$cm_cc_need_credit_plural = esc_attr(get_option('obr_cm_cc_need_credit_plural'));
				if (intval(strlen($cm_cc_need_credit_plural)) == 0){
					$plural = __('You need %d credits to access this page.', 'uc');
				} else {
					$plural = $cm_cc_need_credit_plural;
				}
				$messagestring2 = sprintf(_n($singular, $plural, $pagecost, 'uc'), $pagecost);

				$returnstring .= '<p>'.$messagestring1.'  '.$messagestring2.'</p>';

				if ($credits >= $pagecost){
					// the user has enough credits so give them the option to buy the page
					$cm_cc_need_to_purchase_button = esc_attr(get_option('obr_cm_cc_need_to_purchase_button'));
					if (intval(strlen($cm_cc_need_to_purchase_button)) == 0){
						$buttontext = __('Purchase Page', 'uc');
					} else {
						$buttontext = $cm_cc_need_to_purchase_button;
					}
					$returnstring .= '<form action="" method="post">';
					$returnstring .= '<input type="hidden" name="pagecost" value='.$pagecost.' />';
					$returnstring .= '<input type="hidden" name="page" value='.$pageid.' />';
					$cm_cc_confirm_purchase = esc_attr(get_option('obr_cm_cc_confirm_purchase'));
					if (intval(strlen($cm_cc_confirm_purchase)) == 0){
						$returnstring .= '<input type="submit" name="submit" value="'.$buttontext.'" class="page-button red-button" />';
					} else {
						$returnstring .= '<input type="submit" name="submit" value="'.$buttontext.'" class="page-button red-button" onclick="return confirm(\''.$cm_cc_confirm_purchase.'\')" />';
					}
					$returnstring .= '</form>';
				} else {
					// the user doesn't have enough credits so they need to buy more
					$cm_cc_need_more_credits = esc_attr(get_option('obr_cm_cc_need_more_credits'));
					$cm_cc_need_more_credits_link = esc_url(get_option('obr_cm_cc_need_more_credits_link'));
					if (intval(strlen($cm_cc_need_more_credits)) == 0){
						$message = __('You need to buy more credits to access this page.', 'uc');
					} else {
						$message = $cm_cc_need_more_credits;
					}
					if (strlen($cm_cc_need_more_credits_link) > 0){
						$returnstring .= '<p><a href="'.$cm_cc_need_more_credits_link.'">'.$message.'</a></p>';
					} else {
						$returnstring .= '<p>'.$message.'</p>';
					}
				}
				$returnstring .= '</div>';
			}
			return $returnstring;
		}

		function obr_balance($userid = 0){
			// what's the user's balance?
			global $wpdb;
			global $obr_table_name;
			$credits = intval($wpdb->get_var("select fld_credits from $obr_table_name where ID=$userid;"));
			return $credits;
		}

		function obr_expirydate($userid = 0){
			// what's the expiry date?
			global $wpdb;
			global $obr_table_name;
			$expirydate = esc_attr($wpdb->get_var("select fld_expiry_date from $obr_table_name where ID=$userid;"));
			if ($expirydate == '0000-00-00 00:00:00'){
				$expirydate = '';
			}
			if ($expirydate !== null && $expirydate !== ''){
				// we do have an expiry so we need to check whether the credits are valid
				$expirydatetime = new DateTime($expirydate, new DateTimezone(get_option('timezone_string')));
				$formattednewexpirydate = $expirydatetime->format('Y-m-d H:i:s');
				$now = new DateTime();
				$now = $now->setTimezone(new DateTimezone(get_option('timezone_string')));
				if ($expirydatetime < $now){
					// the credits have expired so we need to reset them to zero
					$credits = $this->obr_balance($userid);
					$bundletype = intval($wpdb->get_var("select fld_bundle_type from $obr_table_name where ID=$userid;"));
					$expirydate = $this->obr_get_new_expiry_date($bundletype);
					$transactionid = sprintf(__('%1$d credits expired at %2$s', 'uc'), $credits, $formattednewexpirydate);
					$q = $this->obr_update_credits($userid, 0, $bundletype, $expirydate, 1, $transactionid);
					$credits = 0;
					$expirydate = '';
				}
			}
			return $expirydate;
		}

		function obr_highest_balance(){
			// what's the highest balance of any user?
			global $wpdb;
			global $obr_table_name;
			$q = $wpdb->get_row("select ID, fld_credits from $obr_table_name order by fld_credits desc, ID asc limit 1;");
			$returnstring = array('ID' => $q->ID, 'credits' => $q->fld_credits);
			return $returnstring;
		}

		function obr_latest_purchase(){
			// what's the most recent credit purchase?  This may include positive admin adjustments
			global $wpdb;
			global $obr_transaction_log_table_name;
			$q = $wpdb->get_row("select ID, credits, credit_time from $obr_transaction_log_table_name where credits > 0 order by credit_time desc, credits desc, ID asc limit 1;");
			$returnstring = array('ID' => $q->ID, 'credits' => $q->credits, 'credit_time' => $q->credit_time);
			return $returnstring;
		}

		function obr_display_balance($atts, $content = null){
			// display the user's balance
			extract(shortcode_atts(array(	'short' => false, 
											'singular' => null, 
											'plural' => null,
											'showexpiry' => false // N/A if short is true
										), $atts));
			if ($short === 'false'){
				// parameters are passed as strings
				$short = false;
			}
			$short = (bool)$short;
			if ($showexpiry === 'false'){
				// parameters are passed as strings
				$showexpiry = false;
			}
			$showexpiry = (bool)$showexpiry;

			$userid = get_current_user_id();
			$returnstring = '';
			if ($userid){
				$expirydate = '';
				if ($showexpiry){
					// we want to show an expiry date, but we need to check whether there is one.
					$expirydate = $this->obr_expirydate($userid);
					if ($expirydate == ''){
						// we don't have an expiry date, so treat it as though there is no expiry date
						$showexpiry = false;
					}
				}
				$credits = $this->obr_balance($userid);

				if ($singular === null || strlen($singular) == 0){
					if ($showexpiry){
						// we need to include an expiry date
						$cm_db_singular_with_expiry = esc_attr(get_option('obr_cm_db_singular_with_expiry'));
						if (intval(strlen($cm_db_singular_with_expiry)) == 0){
							$singular = __('You have 1 credit which expires on %s.', 'uc');
						} else {
							$singular = $cm_db_singular_with_expiry;
						}
					} else {
						$cm_db_singular = esc_attr(get_option('obr_cm_db_singular'));
						if (intval(strlen($cm_db_singular)) == 0){
							$singular = __('You have 1 credit.', 'uc');
						} else {
							$singular = $cm_db_singular;
						}
					}
				}
				if ($plural === null || strlen($plural) == 0){
					if ($showexpiry){
						// we need to include an expiry date
						$cm_db_plural_with_expiry = esc_attr(get_option('obr_cm_db_plural_with_expiry'));
						if (intval(strlen($cm_db_plural_with_expiry)) == 0){
							$plural = __('You have %d credits which expire on %s.', 'uc');
						} else {
							$plural = $cm_db_plural_with_expiry;
						}
					} else {
						$cm_db_plural = esc_attr(get_option('obr_cm_db_plural'));
						if (intval(strlen($cm_db_plural)) == 0){
							$plural = __('You have %d credits.', 'uc');
						} else {
							$plural = $cm_db_plural;
						}
					}
				}

				if ($short){
					// we want the short version
					$returnstring = $credits;
				} else {
					// we want the long version
					$returnstring = '<p>'.sprintf(_n($singular, $plural, $credits, 'uc'), $credits, $expirydate).'</p>';
				}
			} else {
				$cm_db_not_logged_in = esc_attr(get_option('obr_cm_db_not_logged_in'));
				if (intval(strlen($cm_db_not_logged_in)) == 0){
					$returnstring = __('You need to be logged in to view your balance.', 'uc');
				} else {
					$returnstring = $cm_db_not_logged_in;
				}
			}
			return $returnstring.$content;
		}

		function obr_admin_bar_item($wp_admin_bar){
			// display the user's balance in the admin bar
			$enable_admin_bar_balance = get_option('obr_enable_admin_bar_balance', 'on');
			$enable_admin_bar_balance = (bool)$enable_admin_bar_balance;
			if ($enable_admin_bar_balance){
				$userid = get_current_user_id();
				$credits = $this->obr_balance($userid);
				$display = sprintf(_n('My UserCredits balance: 1 credit', 'My UserCredits balance: %d credits', $credits, 'uc'), $credits);
				$href = 'admin.php?page=obr-usercredits';
				if (!is_admin()){
					// this is not an admin page so need to add a prefix
					$href = get_admin_url().$href;
				}
				$args = array(
							'id'    => 'obr_balance',
							'title' => $display,
							'href'  => $href,
							'meta'  => array('class' => 'obr_balance_link')
						);
				$wp_admin_bar->add_node($args);
			}
		}

		function obr_add_credits($atts, $content = null){
			// add credits to a user's balance using the shortcode
			global $wpdb;
			global $obr_transaction_log_table_name;
			
			$userid = get_current_user_id();
			if ($userid <= 0){
				return false;
			}
			$default_add_variable_name = get_option('obr_default_add_variable_name');
			$default_transaction_variable_name = get_option('obr_default_transaction_variable_name');
			// assume a valid transaction
			$valid_txn = true;
			// get results depending on the method set by the user, e.g. (1) $_POST (default), (2) $_GET or (3) $_REQUEST
			$credits = 0;
			$transactionid = '';
			$processing_method = intval(get_option('obr_processing_method', '1'));
			switch ($processing_method){
				case 1:
					if (isset($_POST[$default_add_variable_name]) && isset($_POST[$default_transaction_variable_name])){
						$credits = floatval($_POST[$default_add_variable_name]);
						$transactionid = esc_attr($_POST[$default_transaction_variable_name]);
					}
					break;
				case 2:
					if (isset($_GET[$default_add_variable_name]) && isset($_GET[$default_transaction_variable_name])){
						$credits = floatval($_GET[$default_add_variable_name]);
						$transactionid = esc_attr($_GET[$default_transaction_variable_name]);
					}
					break;
				case 3:
					if (isset($_REQUEST[$default_add_variable_name]) && isset($_REQUEST[$default_transaction_variable_name])){
						$credits = floatval($_REQUEST[$default_add_variable_name]);
						$transactionid = esc_attr($_REQUEST[$default_transaction_variable_name]);
					}
					break;
			}

			if (($credits > 0) && (strlen($transactionid) > 0)){
				// check to see if we've had this amount before
				$match = intval($wpdb->get_var("SELECT fld_ref FROM $obr_transaction_log_table_name WHERE transaction_id='$transactionid';"));
				if ($match){
					// we've had this transaction before so stop here
					$cm_dup_txn = esc_attr(get_option('obr_cm_dup_txn'));
					if (strlen($cm_dup_txn) == 0){
						$cm_dup_txn = __('Transaction ID used previously.', 'uc');
					}
					$returnstring = $cm_dup_txn;
					$valid_txn = false;
				}
			} else {
				$cm_nothing_to_process = esc_attr(get_option('obr_cm_nothing_to_process'));
				if (strlen($cm_nothing_to_process) == 0){
					$cm_nothing_to_process = __('Nothing to process.  If you think you are seeing this message in error, please contact the administrator.', 'uc');
				}
				$returnstring = $cm_nothing_to_process;
				$valid_txn = false;
			}

			if ($valid_txn === false){
				return $returnstring;
			}
			$processcredits = $this->obr_process_add_credits($userid, $credits, $transactionid);
			$message = '<p>'.$processcredits['message'].'</p>';
			if ($processcredits['valid'] === false){
				return $message;
			}
			return $message.$content;
		}

		function obr_process_add_credits($userid, $credits, $transactionid){
			// this is the handler for processing the adding of credits
			global $wpdb;
			global $obr_table_name;

			$result = $this->obr_check_valid_bundle($credits);
			$result['credits'] = intval($result['credits']);
			if ($result['credits'] > 0){
				$result['ref'] = intval($result['ref']);
				$newexpiry = $this->obr_get_new_expiry_date($result['ref']);

				$oldbalance = intval($wpdb->get_var("select fld_credits from $obr_table_name where ID=$userid;"));
				$newbalance = $oldbalance + $result['credits'];

				// okay, we're all set to increase the credits, so let's see whether there's a user block on it...
				$goodtogo = true;
				$args = array(
							'valid_txn' => $goodtogo, 
							'userid' => $userid,
							'amount' => $credits, 
							'txn_id' => $transactionid, 
							'credits' => $result['credits'],
							'bundle_ref' => $result['ref'],
							'expiry' => $newexpiry
						);
				$process = apply_filters_ref_array('obr_txn_check', array($args));
				$goodtogo = $process['valid_txn'];

				if ($goodtogo){
					$q = $this->obr_update_credits($userid, $newbalance, $result['ref'], $newexpiry, 0, $transactionid);
					if ($q){
						// we've added them successfully
						$cm_inc_success = esc_attr(get_option('obr_cm_inc_success'));
						if (strlen($cm_inc_success) == 0){
							$cm_inc_success = __('Successfully added %d credits.', 'uc');
						}
						$message = sprintf($cm_inc_success, $result['credits']);
						$returnarray = array('valid' => true, 'message' => $message);
							
						// now we process the obr_added_credits hook
						$hookargs = array(
									'userid' => $userid,
									'amount' => $credits, 
									'credits' => $result['credits'],
									'txn_id' => $transactionid, 
									'newbalance' => $newbalance,
									'expiry' => $newexpiry
								);
						do_action('obr_added_credits', $hookargs);
					} else {
						$cm_inc_failure = esc_attr(get_option('obr_cm_inc_failure'));
						if (strlen($cm_inc_failure) == 0){
							$cm_inc_failure = __('Could not record your transaction.  Please contact the administrator.', 'uc');
						}
						$returnarray = array('valid' => false, 'message' => $cm_inc_failure);
					}
				} else {
					$cm_site_txn = esc_attr(get_option('obr_cm_site_txn'));
					if (strlen($cm_site_txn) == 0){
						$cm_site_txn = __('Failed site transaction filter.', 'uc');
					}
					$returnarray = array('valid' => false, 'message' => $cm_site_txn);
				}
			} else {
				$cm_bundle_no_match = esc_attr(get_option('obr_cm_bundle_no_match'));
				if (strlen($cm_bundle_no_match) == 0){
					$cm_bundle_no_match = __('%1$.2f does not match a valid bundle.', 'uc');
				}
				$nomatch = sprintf($cm_bundle_no_match, $credits);
				$returnarray = array('valid' => false, 'message' => $nomatch);
			}
			return $returnarray;
		}

		function obr_update_credits($id, $balance, $bundle = 0, $updateexpiry = 0, $balancereset = 0, $transactionid = null, $userbalanceadminexpiryupdate = -1){
			// key function to update the number of credits
			global $wpdb;
			global $obr_table_name;
			global $obr_transaction_log_table_name;

			$existingbalance = $this->obr_balance($id);
			$change = $balance - $existingbalance;

			$q = $this->obr_log_transaction($id, $change, $transactionid);
			if ($balancereset == 1){
				// reset the balance to zero
				$q2 = $wpdb->update($obr_table_name, 
										array('fld_credits' => 0, 
											  'fld_bundle_type' => $bundle, 
											  'fld_expiry_date' => null,
											  'last_credit' => $this->obr_datetime()	), 
										array('ID' => $id), 
										array('%d', '%d', '%s', '%s'), 
										array('%d'));

			} elseif ($userbalanceadminexpiryupdate >= 0){
				// we've come from the user balances admin page ans want to do a direct expiry date update
				$expirydate = $this->obr_get_new_expiry_date($bundle, $userbalanceadminexpiryupdate);
				$q2 = $wpdb->update($obr_table_name, 
										array('fld_credits' => $balance, 
											  'fld_bundle_type' => $bundle, 
											  'fld_expiry_date' => $expirydate,
											  'last_credit' => $this->obr_datetime()	), 
										array('ID' => $id), 
										array('%d', '%d', '%s', '%s'), 
										array('%d'));

			} elseif ($updateexpiry){
				// we want to update the expiry date
				$expirydate = $this->obr_get_new_expiry_date($bundle);
				$q2 = $wpdb->update($obr_table_name, 
										array('fld_credits' => $balance, 
											  'fld_bundle_type' => $bundle, 
											  'fld_expiry_date' => $expirydate,
											  'last_credit' => $this->obr_datetime()	), 
										array('ID' => $id), 
										array('%d', '%d', '%s', '%s'), 
										array('%d'));

			} elseif ($updateexpiry === null){
				// we want to update the expiry date
				$expirydate = $this->obr_get_new_expiry_date($bundle);
				$q2 = $wpdb->update($obr_table_name, 
										array('fld_credits' => $balance, 
											  'fld_bundle_type' => $bundle, 
											  'fld_expiry_date' => null,
											  'last_credit' => $this->obr_datetime()	), 
										array('ID' => $id), 
										array('%d', '%d', '%s', '%s'), 
										array('%d'));

			} else {
				// just update the credits
				$q2 = $wpdb->update($obr_table_name, 
										array('fld_credits' => $balance, 
											  'fld_bundle_type' => $bundle,
											  'last_credit' => $this->obr_datetime()	), 
										array('ID' => $id), 
										array('%d', '%d', '%s'), 
										array('%d'));
			}

			if ($q && $q2){
				return true;
			}
			return false;
		}

		function obr_check_valid_bundle($value){
			// check to see if this is a valid credit bundle
			global $wpdb;
			global $obr_admin_table_name;
			$q = $wpdb->get_row("select * from $obr_admin_table_name where fld_key=".$this->obr_bundle_key." and fld_value=".$value);
			if (!$q){
				return false;
			}
			$returnarray = array('ref' => $q->fld_ref, 'credits' => $q->fld_value2, 'expiryperiod' => $q->fld_expiry_period);
			return $returnarray;
		}

		function obr_get_new_expiry_date($bundle = 0, $userbalanceadminexpiryupdate = -1){
			// function to return a new expiry date for a given bundle
			global $wpdb;
			global $obr_admin_table_name;

			global $expiryvaluesfordb;

			if ($userbalanceadminexpiryupdate >= 0){
				// we have a user balances admin page direct expiry override

				$newexpirydate = new DateTime();
				$newexpirydate = $newexpirydate->setTimezone(new DateTimezone(get_option('timezone_string')));
				$newexpirydate->format('Y-m-d H:i:s');
				$expiryperiod = $expiryvaluesfordb[$userbalanceadminexpiryupdate];
				if (strlen($expiryperiod) > 0){
					$newexpirydate->modify($expiryperiod);
					$formattednewexpirydate = $newexpirydate->format('Y-m-d H:i:s');
				} else {
					$formattednewexpirydate = null;
				}

			} elseif ($bundle == 0){

				$formattednewexpirydate = null;
			
			} else {

				$formattednewexpirydate = '';
				$bundles = $wpdb->get_results("SELECT * FROM $obr_admin_table_name WHERE fld_key = ".$this->obr_bundle_key." AND fld_ref = $bundle;");
				if ($wpdb->num_rows == 1){
					foreach ($bundles as $bundle){
						$expiry = $bundle->fld_expiry_period;
						if ($expiry == 0){
							return null;
						}
						$newexpirydate = new DateTime();
						$newexpirydate = $newexpirydate->setTimezone(new DateTimezone(get_option('timezone_string')));
						$newexpirydate->format('Y-m-d H:i:s');
						$expiryperiod = $expiryvaluesfordb[$expiry];
						if (strlen($expiryperiod) > 0){
							$newexpirydate->modify($expiryperiod);
							$formattednewexpirydate = $newexpirydate->format('Y-m-d H:i:s');
						} else {
							$formattednewexpirydate = null;
						}
					}
				}

			}
			return $formattednewexpirydate;		
		}

		function obr_update_bundle($ref, $value, $credit, $expiry){
			// update an existing value/credit bundle
			global $wpdb;
			global $obr_admin_table_name;
			$q = $wpdb->update($obr_admin_table_name, 
								array('fld_value' => $value, 
									  'fld_value2' => $credit, 
									  'fld_expiry_period' => $expiry), 
								array('fld_ref' => $ref));
			?>
			<div id="message" class="updated fade"><p><strong>
				<?php _e('Credit bundle updated.', 'uc'); ?>
			</strong></p></div>
			<?php
		}

		function obr_delete_bundle($ref){
			// delete an existing value/credit bundle
			global $wpdb;
			global $obr_admin_table_name;
			$q = $wpdb->delete($obr_admin_table_name,
								array('fld_ref' => $ref),
								array('%d'));
			?>
			<div id="message" class="updated fade"><p><strong>
				<?php _e('Credit bundle deleted.', 'uc'); ?>
			</strong></p></div>
			<?php
		}	

		function obr_add_bundle($value, $credit, $expiry){
			// add a new value/credit bundle
			global $wpdb;
			global $obr_admin_table_name;
			$mysql = $wpdb->get_results("SELECT * FROM $obr_admin_table_name WHERE fld_key = ".$this->obr_bundle_key." AND (fld_value = $value OR fld_value2 = $credit);");
			if ($wpdb->num_rows == 0){
				$obr_add_bundle = $wpdb->insert($obr_admin_table_name, 
												array('fld_key' => $this->obr_bundle_key, 
													  'fld_value' => $value, 
													  'fld_value2' => $credit, 
													  'fld_expiry_period' => $expiry));
				?>
				<div id="message" class="updated fade"><p><strong>
					<?php _e('Credit bundle added.', 'uc'); ?>
				</strong></p></div>
				<?php
			} else {
				$error_string = __('Values and/or credits must be unique to avoid confusion with existing bundles.', 'uc');
				?>
				<div id="message" class="error fade"><p><strong>
					<?php _e('Credit bundle could not be added.', 'uc'); ?>&nbsp;
					<?php echo $error_string; ?>
				</strong></p></div>
				<?php
				return $error_string;
			}
		}

		function obr_add_expiry_select($selected = 0){
			// add in a select option of expiry dates
			global $expiryvalues;
			?>
			<select id='expiry' name='expiry'>
				<?php foreach ($expiryvalues as $key => $expiryvalue) : ?>
					<option value=<?php
					echo $key;
					if ($selected == $key){
						echo ' selected';
					}
					?>>
						<?php echo $expiryvalue; ?>
					</option>
				<?php endforeach;?>
			</select>
			<?php
		}

		function obr_add_bundle_select($selected = 0){
			// add in a select option of bundle options
			global $wpdb;
			global $obr_admin_table_name;
			?>
			<select id='bundle' name='bundle'>
				<?php 
				$bundles = $wpdb->get_results("SELECT * FROM $obr_admin_table_name WHERE fld_key = ".$this->obr_bundle_key.";");
				if ($wpdb->num_rows > 0) : ?>
					<option value=0<?php if ($selected == 0) { echo ' selected'; } ?>><?php _e('None', 'uc'); ?></option>
					<?php foreach ($bundles as $bundle) : ?>
						<option value=<?php
						echo $bundle->fld_ref;
						if ($selected == $bundle->fld_ref){
							echo ' selected';
						}
						?>>
							<?php printf(__('Bundle %1$d (%2$01.2f value - %3$d credits)', 'uc'), $bundle->fld_ref, $bundle->fld_value, $bundle->fld_value2); ?>
						</option>
					<?php endforeach;?>
				<?php endif; ?>
			</select>
			<?php
		}

		function obr_get_default_credits_for_bundle($bundle = 0){
			// function to return the number of credits for a given bundle
			global $wpdb;
			global $obr_admin_table_name;

			if ($bundle == 0){
				return null;
			}
			$creditstoadd = intval($wpdb->get_var("SELECT fld_value2 FROM $obr_admin_table_name WHERE fld_key = {$this->obr_bundle_key} AND fld_ref = $bundle;"));
			return $creditstoadd;
		}

		function obr_periodic_credits_review(){
			// function to reset balances to zero where appropriate
			global $wpdb;
			global $obr_table_name;

			$q = $wpdb->get_results("SELECT {$wpdb->prefix}users.ID, display_name, user_email, fld_ref, fld_credits, fld_bundle_type, fld_expiry_date 
									 FROM {$wpdb->prefix}users 
									 LEFT JOIN $obr_table_name ON {$wpdb->prefix}users.ID = $obr_table_name.ID 
									 WHERE fld_expiry_date <> '0000-00-00' AND fld_expiry_date IS NOT NULL AND fld_expiry_date < NOW() 
									 ORDER BY display_name ASC;");
			$count = 0;
			$balancereset = get_option('obr_balancereset');
			$returnstring = '';
			foreach ($q as $userid){
				$bundletype = intval($userid->fld_bundle_type);
				$expirydate = $this->obr_get_new_expiry_date($bundletype);
				$id = intval($userid->ID);
				$transaction_id = __('Automatic Balance Update', 'uc');

				if ($balancereset == 1){
					// reset the balance to zero
					$newcreditsbalance = 0;
				} else {
					// reset to the default for that bundle
					$newcreditsbalance = $this->obr_get_default_credits_for_bundle($bundletype);
					if ($balancereset == 2){
						// we're adding the default to the current amount
						$newcreditsbalance += $userid->fld_credits;
					}
				}
				$q = $this->obr_update_credits($id, $newcreditsbalance, $bundletype, $expirydate, $balancereset, $transaction_id);

				$returnstring .= '<p>'.sprintf(__('User %1$d (%2$s) updated to %3$d credits.', 'uc'), $userid->ID, $userid->display_name, $newcreditsbalance).'</p>';
				$count++;
			}
			if ($count){
				$message = sprintf(_n(__('1 user was updated.', 'uc'), __('%d user(s) were updated.', 'uc'), $count, 'uc'), $count);
			} else {
				$message = __('No users needed updating.', 'uc');
			}

			$emailmessage = wp_filter_nohtml_kses($message.$returnstring);
			wp_mail(get_bloginfo('admin_email'), sprintf(__('UserCredits on %s', 'uc'), get_bloginfo('name')), $emailmessage);
			?>
			<div id="message" class="updated fade">
				<p><strong>
					<?php echo $message; ?>
				</strong></p>
				<?php echo $returnstring; ?>
			</div>
			<?php
		}

		function obr_add_to_wp_cron_options($schedules){
			$schedules['weekly'] = array(
										'interval' => 7*24*60*60, 
										'display' => __('Once A Week', 'uc')
									);
			return $schedules;
		}

		function obr_schedule_activation(){
			wp_schedule_event(current_time('timestamp'), 'twicedaily', 'obr_event_hook_1');

			wp_schedule_event(current_time('timestamp'), 'hourly', 'obr_event_hook_2');
			wp_schedule_event(current_time('timestamp'), 'daily', 'obr_event_hook_3');
			wp_schedule_event(current_time('timestamp'), 'weekly', 'obr_event_hook_4');
		}

		function obr_schedule_deactivation(){
			wp_clear_scheduled_hook('obr_event_hook_1');
			wp_clear_scheduled_hook('obr_event_hook_2');
			wp_clear_scheduled_hook('obr_event_hook_3');
			wp_clear_scheduled_hook('obr_event_hook_4');
		}

		function obr_log_transaction($userid, $credits, $transactionlogcomment){
			//function to record details of a transaction in the transaction log
			global $wpdb;
			global $obr_transaction_log_table_name;

			$q = $wpdb->insert($obr_transaction_log_table_name,
								array('ID' => $userid, 
									  'credits' => $credits, 
									  'credit_time' => $this->obr_datetime(), 
									  'transaction_id' => $transactionlogcomment),
								array('%d', '%d', '%s', '%s'));
			if (!$q){
				return false;
			}
			return true;
		}

		function obr_uc_shortcode_button_init(){
			// init process for registering our button

			// abort early if the user will never see TinyMCE
			if (!current_user_can('edit_posts') && !current_user_can('edit_pages') && get_user_option('rich_editing') == 'true'){
				return;
			}
			// add a callback to register our TinyMCE plugin
			add_filter('mce_external_plugins', array($this, 'obr_uc_register_tinymce_plugin'));
			// add a callback to add our button to the TinyMCE toolbar
			add_filter('mce_buttons', array($this, 'obr_uc_add_tinymce_button'));
		}

		function obr_uc_register_tinymce_plugin($plugin_array){
			// this callback registers our plug-in
			$plugin_array['obr_uc_buttons'] = plugins_url('user-credits.js', __FILE__);
			return $plugin_array;
		}

		function obr_uc_add_tinymce_button($buttons){
			// this callback adds our button to the toolbar
			$buttons[] = "obr_uc_display_balance_button";
			$buttons[] = "obr_uc_page_button_button";
			$buttons[] = "obr_uc_check_credits_button";
			$buttons[] = "obr_uc_add_credits_button";

			return $buttons;
		}
	}
	// set the new class
	$obr_usercredits = new obr_usercredits;
}

class obr_usercredits_widget extends WP_Widget{
	// widget to display the user's credit balance in the sidebar

	// constructor
	function obr_usercredits_widget(){
		$widget_ops = array(
								'classname' => 'obr_usercredits_widget', 
								'description' => __("Display user's credit balances in the sidebar.  Only displayed to logged in users.", 'uc')
							);
		$control_ops = array('width' => 300, 'height' => 300);
		$this->WP_Widget('obr_usercredits', __('UserCredits - Display User\'s Balance', 'uc'), $widget_ops, $control_ops);
	}
	
	// functions
	function widget($args, $instance){
		// widget output to the sidebar
		global $obr_usercredits;

		$userid = get_current_user_id();
		if ($userid > 0){
			// the user is logged in so we can display the widget
			$credits = $obr_usercredits->obr_balance($userid);

			$title = apply_filters('widget_title', $instance['title']);
			$singular = esc_textarea($instance['singular']);
			$plural = esc_textarea($instance['plural']);
			echo $args['before_widget'];
			if ($title){
				echo $args['before_title'],$title,$args['after_title'];
			} else {
				echo $args['before_title'],__('Your Credits', 'uc'),$args['after_title'];
			}
			if (!$singular){
				$singular = __('You have 1 credit.', 'uc');
			}
			if (!$plural){
				$plural = __('You have %d credits.', 'uc');
			}
			$outputstring = '<p>'.sprintf(_n($singular, $plural, $credits, 'uc'), $credits).'</p>';
			echo $outputstring;
			echo $args['after_widget'];
		}
	}

	function update($new_instance, $old_instance){
		$instance = $old_instance;
		$instance['title'] = strip_tags(stripslashes($new_instance['title']));
		$instance['singular'] = esc_textarea($new_instance['singular']);
		$instance['plural'] = esc_textarea($new_instance['plural']);
		return $instance;
	}
	
	function form($instance){
		$instance = wp_parse_args((array) $instance, array('title' => __('Credits Balance', 'uc')));
		$title = strip_tags(stripslashes($instance['title']));
		$singular = '';
		if (isset($instance['singular'])){
			$singular = esc_textarea($instance['singular']);
		}
		$plural = '';
		if (isset($instance['plural'])){
			$plural = esc_textarea($instance['plural']);
		}
		?>
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:', 'uc'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('singular'); ?>"><?php _e('Singular:', 'uc'); ?></label>
			<textarea class="widefat" rows="4" cols="10" id="<?php echo $this->get_field_id('singular'); ?>" name="<?php echo $this->get_field_name('singular'); ?>"><?php if(!empty($singular)) echo $singular; ?></textarea>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('plural'); ?>"><?php _e('Plural (use %d where you want the actual number of credits to be displayed):', 'uc'); ?></label>
			<textarea class="widefat" rows="4" cols="10" id="<?php echo $this->get_field_id('plural'); ?>" name="<?php echo $this->get_field_name('plural'); ?>"><?php if(!empty($plural)) echo $plural; ?></textarea>
		</p>
		<?php
	}
}

class obr_usercredits_top_bal_widget extends WP_Widget{
	// widget to display the highest user's balance in the sidebar

	// constructor
	function obr_usercredits_top_bal_widget(){
		$widget_ops = array(
								'classname' => 'obr_usercredits_top_bal_widget', 
								'description' => __("Display highest user balance in the sidebar.  Always displayed (regardless of whether user is logged in or not).", 'uc')
							);
		$control_ops = array('width' => 300, 'height' => 300);
		$this->WP_Widget('obr_usercredits_top_bal', __('UserCredits - Display Highest Balance', 'uc'), $widget_ops, $control_ops);
	}
	
	// functions
	function widget($args, $instance){
		// widget output to the sidebar
		global $obr_usercredits;

		$highest = $obr_usercredits->obr_highest_balance();
		$userdata = get_userdata($highest['ID']);
		$who = $userdata->display_name;
		$credits = $highest['credits'];

		$userid = get_current_user_id();

		$title = apply_filters('widget_title', $instance['title']);
		$singular = esc_textarea($instance['singular']);
		$plural = esc_textarea($instance['plural']);
		$topdog = esc_textarea($instance['topdog']);
		echo $args['before_widget'];
		if ($title){
			echo $args['before_title'],$title,$args['after_title'];
		} else {
			echo $args['before_title'],__('Highest Balance', 'uc'),$args['after_title'];
		}
		if (!$singular){
			$singular = __('%s has 1 credit.', 'uc');
		}
		if (!$plural){
			$plural = __('%s has %d credits.', 'uc');
		}
		if (!$topdog){
			$topdog = __('Congratulations!  You\'re the top dog.', 'uc');
		}
		$outputstring = '<p>';
		$outputstring .= sprintf(_n($singular, $plural, $credits, 'uc'), $who, $credits);
		if ($userid == $highest['ID']){
			// the user has the highest balance
			$outputstring .= '<br />'.$topdog;
		}
		$outputstring .= '</p>';
		echo $outputstring;
		echo $args['after_widget'];
	}

	function update($new_instance, $old_instance){
		$instance = $old_instance;
		$instance['title'] = strip_tags(stripslashes($new_instance['title']));
		$instance['singular'] = esc_textarea($new_instance['singular']);
		$instance['plural'] = esc_textarea($new_instance['plural']);
		$instance['topdog'] = esc_textarea($new_instance['topdog']);
		return $instance;
	}
	
	function form($instance){
		$instance = wp_parse_args((array) $instance, array('title' => __('Highest Balance', 'uc')));
		$title = strip_tags(stripslashes($instance['title']));
		$singular = '';
		if (isset($instance['singular'])){
			$singular = esc_textarea($instance['singular']);
		}
		$plural = '';
		if (isset($instance['plural'])){
			$plural = esc_textarea($instance['plural']);
		}
		if (isset($instance['topdog'])){
			$topdog = esc_textarea($instance['topdog']);
		}
		?>
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:', 'uc'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('singular'); ?>"><?php _e('Singular (use %s where you want the user\'s name to be displayed):', 'uc'); ?></label>
			<textarea class="widefat" rows="4" cols="10" id="<?php echo $this->get_field_id('singular'); ?>" name="<?php echo $this->get_field_name('singular'); ?>"><?php if(!empty($singular)) echo $singular; ?></textarea>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('plural'); ?>"><?php _e('Plural (use %1$s where you want the user\'s name to be displayed and %2$d where you want the actual number of credits to be displayed):', 'uc'); ?></label>
			<textarea class="widefat" rows="4" cols="10" id="<?php echo $this->get_field_id('plural'); ?>" name="<?php echo $this->get_field_name('plural'); ?>"><?php if(!empty($plural)) echo $plural; ?></textarea>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('topdog'); ?>"><?php _e('Top dog message, i.e. message to be displayed if the current user also has the highest balance:', 'uc'); ?></label>
			<textarea class="widefat" rows="4" cols="10" id="<?php echo $this->get_field_id('topdog'); ?>" name="<?php echo $this->get_field_name('topdog'); ?>"><?php if(!empty($topdog)) echo $topdog; ?></textarea>
		</p>
		<?php
	}
}

class obr_usercredits_latest_purchase_widget extends WP_Widget{
	// widget to display the most recent credit purchase

	// constructor
	function obr_usercredits_latest_purchase_widget(){
		$widget_ops = array(
								'classname' => 'obr_usercredits_latest_purchase_widget', 
								'description' => __("Display the user with the most recent credits added in the sidebar.  Always displayed (regardless of whether user is logged in or not).", 'uc')
							);
		$control_ops = array('width' => 300, 'height' => 300);
		$this->WP_Widget('obr_usercredits_latest_purchase', __('UserCredits - Display Latest Credits Added', 'uc'), $widget_ops, $control_ops);
	}
	
	// functions
	function widget($args, $instance){
		// widget output to the sidebar
		global $obr_usercredits;

		$highest = $obr_usercredits->obr_latest_purchase();
		$userdata = get_userdata($highest['ID']);
		$who = $userdata->display_name;
		$credits = $highest['credits'];
		$credit_time = $highest['credit_time'];

		$title = apply_filters('widget_title', $instance['title']);
		$singular = esc_textarea($instance['singular']);
		$plural = esc_textarea($instance['plural']);
		echo $args['before_widget'];
		if ($title){
			echo $args['before_title'],$title,$args['after_title'];
		} else {
			echo $args['before_title'],__('Most Recent Credits Added', 'uc'),$args['after_title'];
		}
		if (!$singular){
			$singular = __('%1$s received 1 credit at %3$s.', 'uc');
		}
		if (!$plural){
			$plural = __('%1$s received %2$d credits at %3$s.', 'uc');
		}
		$outputstring = '<p>'.sprintf(_n($singular, $plural, $credits, 'uc'), $who, $credits, $credit_time).'</p>';
		echo $outputstring;
		echo $args['after_widget'];
	}

	function update($new_instance, $old_instance){
		$instance = $old_instance;
		$instance['title'] = strip_tags(stripslashes($new_instance['title']));
		$instance['singular'] = esc_textarea($new_instance['singular']);
		$instance['plural'] = esc_textarea($new_instance['plural']);
		return $instance;
	}
	
	function form($instance){
		$instance = wp_parse_args((array) $instance, array('title' => __('Most Recent Credits Added', 'uc')));
		$title = strip_tags(stripslashes($instance['title']));
		$singular = '';
		if (isset($instance['singular'])){
			$singular = esc_textarea($instance['singular']);
		}
		$plural = '';
		if (isset($instance['plural'])){
			$plural = esc_textarea($instance['plural']);
		}
		?>
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:', 'uc'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('singular'); ?>"><?php _e('Singular (use %1$s where you want the user\'s name to be displayed.  Use %3$s for the time of the purchase):', 'uc'); ?></label>
			<textarea class="widefat" rows="4" cols="10" id="<?php echo $this->get_field_id('singular'); ?>" name="<?php echo $this->get_field_name('singular'); ?>"><?php if(!empty($singular)) echo $singular; ?></textarea>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('plural'); ?>"><?php _e('Plural (use %1$s where you want the user\'s name to be displayed.  Use %2$d for the number of credits purchased and %3$s for the time of the purchase):', 'uc'); ?></label>
			<textarea class="widefat" rows="4" cols="10" id="<?php echo $this->get_field_id('plural'); ?>" name="<?php echo $this->get_field_name('plural'); ?>"><?php if(!empty($plural)) echo $plural; ?></textarea>
		</p>
		<?php
	}
}

function obr_widget_init(){
	register_widget('obr_usercredits_widget');
	register_widget('obr_usercredits_top_bal_widget');
	register_widget('obr_usercredits_latest_purchase_widget');
}
add_action('widgets_init', 'obr_widget_init');

?>